package com.mortals.xhx.module.staff.web;

import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.model.ParamEntity;
import com.mortals.xhx.base.system.param.model.ParamQuery;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.CareTypeEnum;
import com.mortals.xhx.common.code.SendStatusEnum;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.job.model.JobEntity;
import com.mortals.xhx.module.job.model.JobQuery;
import com.mortals.xhx.module.job.service.JobService;
import com.mortals.xhx.module.staff.model.StaffCareQuery;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.model.StaffQuery;
import com.mortals.xhx.module.staff.service.StaffService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.staff.model.StaffCareEntity;
import com.mortals.xhx.module.staff.service.StaffCareService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;
import static com.mortals.xhx.common.key.ParamKey.*;

/**
 * 员工关怀信息
 *
 * @author zxfei
 * @date 2023-04-07
 */
@RestController
@RequestMapping("staff/care")
public class StaffCareController extends BaseCRUDJsonBodyMappingController<StaffCareService, StaffCareEntity, Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private StaffService staffService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private JobService jobService;


    public StaffCareController() {
        super.setModuleDesc("员工关怀信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "careType", CareTypeEnum.getEnumMap());
        this.addDict(model, "sendStatus", SendStatusEnum.getEnumMap());
        this.addDict(model, "deptId", deptService.find(new DeptQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName())));
        this.addDict(model, "jobId", jobService.find(new JobQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getJobName())));
        this.addDict(model, "staffId", staffService.find(new StaffQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getName())));
        this.addDict(model, "staffType", paramService.getParamBySecondOrganize("Staff", "staffType"));
        super.init(model, context);
    }


    @Override
    protected void saveBefore(StaffCareEntity entity, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getJobId())) {
            JobEntity jobEntity = jobService.get(entity.getJobId(), context);
            entity.setJobName(jobEntity == null ? "" : jobEntity.getJobName());
        }

        if (!ObjectUtils.isEmpty(entity.getStaffId())) {
            StaffEntity staffEntity = staffService.getCache(entity.getStaffId().toString());
            entity.setStaffName(staffEntity == null ? "" : staffEntity.getName());
            entity.setDeptId(staffEntity == null ? null : staffEntity.getDeptId());
            entity.setStaffType(staffEntity == null ? null : staffEntity.getStaffType());
        }
        if (!ObjectUtils.isEmpty(entity.getDeptId())) {
            DeptEntity deptEntity = deptService.get(entity.getDeptId(), context);
            entity.setDeptName(deptEntity == null ? "" : deptEntity.getDeptName());
        }

    }

    @Override
    protected void doListBefore(StaffCareEntity query, Map<String, Object> model, Context context) throws AppException {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH)+1;
        if(query.getYear()==null){
            query.setYear(year);
        }
        if(query.getMonth()==null){
            query.setMonth(month);
        }
//        YearMonth yearMonth = YearMonth.of(year,month);
//        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
//        if(query.getCareType()==1){
//            query.setBirthdayStart(yearMonth.atDay(1).format(formatter));
//            query.setBirthdayEnd(yearMonth.atEndOfMonth().format(formatter));
//        }
//        if(query.getCareType()==2){
//            query.setEntryDateStart(yearMonth.atDay(1).format(formatter));
//            query.setEntryDateEnd(yearMonth.atEndOfMonth().format(formatter));
//        }
    }


    @Override
    protected int doListAfter(StaffCareEntity query, Map<String, Object> model, Context context) throws AppException {
        query.setCareType(1);
        int shengri = this.service.getStaffCareListCount(query);
        query.setCareType(2);
        int ruzhi = this.service.getStaffCareListCount(query);
        model.put("shengri", shengri);
        model.put("ruzhi", ruzhi);
        model.put(STAFF_CARE_BIRTHDAY_SEND, paramService.getValueByKey(STAFF_CARE_BIRTHDAY_SEND));
        model.put(STAFF_CARE_ENTRYDATE_SEND, paramService.getValueByKey(STAFF_CARE_ENTRYDATE_SEND));
        return 1;
    }

    /**
     * 设置关怀发送开关
     * @param query
     * @return
     */
    @PostMapping({"sendSwitch"})
    public String sendSwitch(@RequestBody StaffCareEntity query) {
        Map<String, Object> model = new HashMap();
        int code = 1;
        String busiDesc = "设置关怀发送开关";
        try {
            if(StringUtils.isNotEmpty(query.getParamType())){
                ParamQuery paramQuery = new ParamQuery();
                paramQuery.setParamKey(query.getParamType());
                ParamEntity paramEntity = paramService.selectOne(paramQuery);
                if(paramEntity!=null){
                    paramEntity.setParamValue(String.valueOf(query.getParamValue()));
                    paramService.update(paramEntity);
                }
            }

            model.put("message_info", busiDesc + " 【成功】");
            this.recordSysLog(this.request, busiDesc + " 【成功】 ");
        } catch (Exception var7) {
            code = -1;
            this.doException(this.request, busiDesc, model, var7);
        }

        JSONObject ret = new JSONObject();
        ret.put("code", Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        ret.put("data", model);
        return ret.toJSONString();
    }

}